//=============================================================================
//
//  CLASS DecodingViewer - IMPLEMENTATION
//
//=============================================================================


//== INCLUDES =================================================================

#include <OpenMesh/Core/IO/MeshIO.hh>

#include "DecodingViewer.hh"

#include <fstream>
#include <string>

using namespace std;

//== IMPLEMENTATION ========================================================== 


DecodingViewer::
DecodingViewer(const char* _title, int _width, int _height, const char* _inputfile, const char* _outputfile)
  : MeshViewer(_title, _width, _height)
{ 
  _InputFileName = _inputfile;
  _OutputFileName = _outputfile;

  mesh_.request_vertex_colors();

  mesh_.add_property(_nFreeEdges);

	init();
}

//-----------------------------------------------------------------------------

void 
DecodingViewer::
draw(const std::string& _draw_mode)
{
  if (!indices_.empty())
  {
    MeshViewer::draw(_draw_mode);
  }
}

//-----------------------------------------------------------------------------

bool
DecodingViewer::
update_scene()
{
  // set center and radius
  Mesh::ConstVertexIter  v_it(mesh_.vertices_begin()),  v_end(mesh_.vertices_end());
  Mesh::Point bbMin, bbMax;

  bbMin = bbMax = mesh_.point(v_it);
  for (; v_it!=v_end; ++v_it)
  {
    bbMin.minimize(mesh_.point(v_it));
    bbMax.maximize(mesh_.point(v_it));
  }
  set_scene( (Vec3f)(bbMin + bbMax)*0.5, 0.5*(bbMin - bbMax).norm());


  // compute face & vertex normals
  mesh_.update_normals();


  // update face indices for faster rendering
  update_face_indices();

  // info
  std::cerr << mesh_.n_vertices() << " vertices, "
    << mesh_.n_faces()    << " faces\n";

  return true;
}


//-----------------------------------------------------------------------------

DecodingViewer::
~DecodingViewer()
{ 
  _InputFileName = NULL;
  _OutputFileName = NULL;
}

//-----------------------------------------------------------------------------

void DecodingViewer::
decode()
{
  // Exercise 7.1 ----------------------------------------------
  // Simple Decoder
  // INSERT CODE:
  // Open the file called _InputFileName and progressively add vertices 
  // and faces to the mesh_ triangle mesh.
  // Use the vertex property _nFreeEdges to keep count of the number
  // of free edges of each vertex of the cut-border
  // Hint: you can use a list of vertex handles (std::list<Mesh::VertexHandle>)
  // as data structure for the cut-border.
  // -----------------------------------------------------------
}

//-----------------------------------------------------------------------------

void
DecodingViewer::
write_off()
{
  // Exercise 7.2 ----------------------------------------------
  // INSERT CODE:
  // Write in the file called _OutputFileName the description of
  // mesh_ in the OFF format
  // -----------------------------------------------------------
}

//-----------------------------------------------------------------------------

void
DecodingViewer::
keyboard(int key, int x, int y)
{
	switch (toupper(key))
	{
	case 'D':
		{
      if (indices_.empty()) // decoder has not been run yet
      {
        std::cout << "Decoding input file: " << std::flush;
        decode();
        write_off();
        update_scene();
        glutPostRedisplay();
        std::cout << "done\n";      
      }
      break;
		}
	default:
		{
			MeshViewer::keyboard(key, x, y);
			break;
		}
	}
}

//-----------------------------------------------------------------------------

//=============================================================================
